@@ -23,6 +23,10 @@ use super::auth::AuthMethod;
 #[cfg(feature = "compute")]
 use super::compute::{Flavor, FlavorQuery, FlavorSummary,
                      Server, ServerQuery, ServerSummary};
+#[cfg(feature = "image")]
+use super::image::{Image, ImageQuery};
+#[cfg(feature = "network")]
+use super::network::{Network, NetworkQuery};
 use super::session::Session;
 #[allow(unused_imports)]
 use super::utils::ResultExt;
@@ -51,26 +55,8 @@ impl Cloud {
     /// let os = openstack::Cloud::new(auth);
     /// ```
     pub fn new<Auth: AuthMethod + 'static>(auth_method: Auth) -> Cloud {
-        Cloud::new_with_session(Session::new(auth_method))
-    }
-
-    /// Create a new cloud object with a given session.
-    ///
-    /// This constructor can be used to modify `Session` parameters before
-    /// using it in the `Cloud` object. This is an advanced feature and
-    /// should generally be avoided.
-    ///
-    /// # Example
-    ///
-    /// ```rust,no_run
-    /// use openstack;
-    ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let session = openstack::session::Session::new(auth);
-    /// let os = openstack::Cloud::new_with_session(session);
-    pub fn new_with_session(session: Session) -> Cloud {
         Cloud {
-            session: session
+            session: Session::new(auth_method)
         }
     }
 
@@ -82,14 +68,136 @@ impl Cloud {
         }
     }
 
+    /// Refresh this `Cloud` object (renew token, refetch service catalog, etc).
+    pub fn refresh(&mut self) -> Result<()> {
+        self.session.auth_method_mut().refresh()
+    }
+
     /// `Session` used with this `Cloud` object.
     pub fn session(&self) -> &Session {
         &self.session
     }
 
-    /// Extract the `Session` object, destroying this `Cloud`.
-    pub fn into_session(self) -> Session {
-        self.session
+    /// Build a query against flavor list.
+    ///
+    /// The returned object is a builder that should be used to construct
+    /// the query.
+    #[cfg(feature = "compute")]
+    pub fn find_flavors(&self) -> FlavorQuery {
+        FlavorQuery::new(&self.session)
+    }
+
+    /// Build a query against image list.
+    ///
+    /// The returned object is a builder that should be used to construct
+    /// the query.
+    #[cfg(feature = "image")]
+    pub fn find_images(&self) -> ImageQuery {
+        ImageQuery::new(&self.session)
+    }
+
+    /// Build a query against network list.
+    ///
+    /// The returned object is a builder that should be used to construct
+    /// the query.
+    #[cfg(feature = "network")]
+    pub fn find_networks(&self) -> NetworkQuery {
+        NetworkQuery::new(&self.session)
+    }
+
+    /// Build a query against server list.
+    ///
+    /// The returned object is a builder that should be used to construct
+    /// the query.
+    ///
+    /// # Example
+    ///
+    /// Sorting servers by `access_ip_v4` and getting first 5 results:
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
+    /// let os = openstack::Cloud::new(auth);
+    /// let sorting = openstack::compute::ServerSortKey::AccessIpv4;
+    /// let server_list = os.find_servers()
+    ///     .sort_by(openstack::Sort::Asc(sorting)).with_limit(5)
+    ///     .all().expect("Unable to fetch servers");
+    /// ```
+    #[cfg(feature = "compute")]
+    pub fn find_servers(&self) -> ServerQuery {
+        ServerQuery::new(&self.session)
+    }
+
+    /// Find a flavor by its name or ID.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
+    /// let os = openstack::Cloud::new(auth);
+    /// let server = os.get_flavor("m1.medium").expect("Unable to get a flavor");
+    /// ```
+    #[cfg(feature = "compute")]
+    pub fn get_flavor<Id: Into<String>>(&self, id_or_name: Id) -> Result<Flavor> {
+        let s = id_or_name.into();
+        Flavor::new(&self.session, &s).if_not_found_then(|| {
+            self.find_flavors().into_iter()
+                .filter(|item| item.name() == &s).take(2)
+                .collect::<Vec<FlavorSummary>>().and_then(|mut items| {
+                    if items.len() > 1 {
+                        Err(Error::new(ErrorKind::TooManyItems,
+                                       "Too many flavors with this name"))
+                    } else {
+                        match items.pop() {
+                            Some(item) => item.details(),
+                            None => Err(Error::new(
+                                ErrorKind::ResourceNotFound,
+                                "No flavors with this name or ID"))
+                        }
+                    }
+                })
+        })
+    }
+
+    /// Find an image by its name or ID.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
+    /// let os = openstack::Cloud::new(auth);
+    /// let server = os.get_image("centos7").expect("Unable to get a image");
+    /// ```
+    #[cfg(feature = "image")]
+    pub fn get_image<Id: Into<String>>(&self, id_or_name: Id) -> Result<Image> {
+        let s = id_or_name.into();
+        Image::new(&self.session, &s).if_not_found_then(|| {
+            self.find_images().with_name(s).one()
+        })
+    }
+
+    /// Find an network by its name or ID.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
+    /// let os = openstack::Cloud::new(auth);
+    /// let server = os.get_network("centos7").expect("Unable to get a network");
+    /// ```
+    #[cfg(feature = "network")]
+    pub fn get_network<Id: Into<String>>(&self, id_or_name: Id) -> Result<Network> {
+        let s = id_or_name.into();
+        Network::new(&self.session, &s).if_not_found_then(|| {
+            self.find_networks().with_name(s).one()
+        })
     }
 
     /// Find a server by its name or ID.
@@ -126,28 +234,64 @@ impl Cloud {
         })
     }
 
-    /// Build a query against server list.
+    /// List all flavors.
     ///
-    /// The returned object is a builder that should be used to construct
-    /// the query.
+    /// This call can yield a lot of results, use the
+    /// [find_flavors](#method.find_flavors) call to limit the number of
+    /// flavors to receive.
     ///
     /// # Example
     ///
-    /// Sorting servers by `access_ip_v4` and getting first 5 results:
-    ///
     /// ```rust,no_run
     /// use openstack;
     ///
     /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
     /// let os = openstack::Cloud::new(auth);
-    /// let sorting = openstack::compute::ServerSortKey::AccessIpv4;
-    /// let server_list = os.find_servers()
-    ///     .sort_by(openstack::Sort::Asc(sorting)).with_limit(5)
-    ///     .all().expect("Unable to fetch servers");
+    /// let server_list = os.list_flavors().expect("Unable to fetch flavors");
     /// ```
     #[cfg(feature = "compute")]
-    pub fn find_servers(&self) -> ServerQuery {
-        ServerQuery::new(&self.session)
+    pub fn list_flavors(&self) -> Result<Vec<FlavorSummary>> {
+        self.find_flavors().all()
+    }
+
+    /// List all images.
+    ///
+    /// This call can yield a lot of results, use the
+    /// [find_images](#method.find_images) call to limit the number of
+    /// images to receive.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
+    /// let os = openstack::Cloud::new(auth);
+    /// let server_list = os.list_images().expect("Unable to fetch images");
+    /// ```
+    #[cfg(feature = "image")]
+    pub fn list_images(&self) -> Result<Vec<Image>> {
+        self.find_images().all()
+    }
+
+    /// List all networks.
+    ///
+    /// This call can yield a lot of results, use the
+    /// [find_networks](#method.find_networks) call to limit the number of
+    /// networks to receive.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
+    /// let os = openstack::Cloud::new(auth);
+    /// let server_list = os.list_networks().expect("Unable to fetch networks");
+    /// ```
+    #[cfg(feature = "network")]
+    pub fn list_networks(&self) -> Result<Vec<Network>> {
+        self.find_networks().all()
     }
 
     /// List all servers.
@@ -169,73 +313,19 @@ impl Cloud {
     pub fn list_servers(&self) -> Result<Vec<ServerSummary>> {
         self.find_servers().all()
     }
-
-    /// Find a flavor by its name or ID.
-    ///
-    /// # Example
-    ///
-    /// ```rust,no_run
-    /// use openstack;
-    ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let os = openstack::Cloud::new(auth);
-    /// let server = os.get_flavor("m1.medium").expect("Unable to get a flavor");
-    /// ```
-    #[cfg(feature = "compute")]
-    pub fn get_flavor<Id: Into<String>>(&self, id_or_name: Id) -> Result<Flavor> {
-        let s = id_or_name.into();
-        Flavor::new(&self.session, &s).if_not_found_then(|| {
-            self.find_flavors().into_iter()
-                .filter(|item| item.name() == &s).take(2)
-                .collect::<Vec<FlavorSummary>>().and_then(|mut items| {
-                    if items.len() > 1 {
-                        Err(Error::new(ErrorKind::TooManyItems,
-                                       "Too many flavors with this name"))
-                    } else {
-                        match items.pop() {
-                            Some(item) => item.details(),
-                            None => Err(Error::new(
-                                ErrorKind::ResourceNotFound,
-                                "No flavors with this name or ID"))
-                        }
-                    }
-                })
-        })
-    }
-
-    /// Build a query against flavor list.
-    ///
-    /// The returned object is a builder that should be used to construct
-    /// the query.
-    #[cfg(feature = "compute")]
-    pub fn find_flavors(&self) -> FlavorQuery {
-        FlavorQuery::new(&self.session)
-    }
-
-    /// List all flavors.
-    ///
-    /// This call can yield a lot of results, use the
-    /// [find_flavors](#method.find_flavors) call to limit the number of
-    /// flavors to receive.
-    ///
-    /// # Example
-    ///
-    /// ```rust,no_run
-    /// use openstack;
-    ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let os = openstack::Cloud::new(auth);
-    /// let server_list = os.list_flavors().expect("Unable to fetch flavors");
-    /// ```
-    #[cfg(feature = "compute")]
-    pub fn list_flavors(&self) -> Result<Vec<FlavorSummary>> {
-        self.find_flavors().all()
-    }
 }
 
 
 impl From<Session> for Cloud {
     fn from(value: Session) -> Cloud {
-        Cloud::new_with_session(value)
+        Cloud {
+            session: value
+        }
+    }
+}
+
+impl From<Cloud> for Session {
+    fn from(value: Cloud) -> Session {
+        value.session
     }
 }
