@@ -22,11 +22,10 @@ use reqwest::header::Headers;
 use serde::Serialize;
 use serde_json;
 
-use super::super::{Result, ApiVersion};
+use super::super::Result;
 use super::super::auth::AuthMethod;
-use super::super::common;
-use super::super::service::{ApiVersioning, ServiceInfo, ServiceType};
-use super::super::session::Session;
+use super::super::common::{self, ApiVersion};
+use super::super::session::{Session, ServiceInfo, ServiceType};
 use super::protocol;
 
 
@@ -37,7 +36,7 @@ pub trait V2API {
 
     /// List servers.
     fn list_servers<Q: Serialize + Debug>(&self, query: &Q)
-        -> Result<Vec<protocol::ServerSummary>>;
+        -> Result<Vec<common::protocol::IdAndName>>;
 
     /// List servers with details.
     fn list_servers_detail<Q: Serialize + Debug>(&self, query: &Q)
@@ -45,7 +44,12 @@ pub trait V2API {
 
     /// Run an action on the server.
     fn server_simple_action<S1, S2>(&self, id: S1, action: S2) -> Result<()>
-        where S1: AsRef<str>, S2: AsRef<str>;
+            where S1: AsRef<str>, S2: AsRef<str> {
+        self.server_action_with_args(id, action, serde_json::Value::Null)
+    }
+
+    fn server_action_with_args<S1, S2, Q>(&self, id: S1, action: S2, args: Q)
+        -> Result<()> where S1: AsRef<str>, S2: AsRef<str>, Q: Serialize + Debug;
 
     /// Delete a server.
     fn delete_server<S: AsRef<str>>(&self, id: S) -> Result<()>;
@@ -55,7 +59,7 @@ pub trait V2API {
 
     /// List flavors.
     fn list_flavors<Q: Serialize + Debug>(&self, query: &Q)
-        -> Result<Vec<protocol::FlavorSummary>>;
+        -> Result<Vec<common::protocol::IdAndName>>;
 
     /// List flavors with details.
     fn list_flavors_detail<Q: Serialize + Debug>(&self, query: &Q)
@@ -73,16 +77,18 @@ const VERSION_ID: &'static str = "v2.1";
 impl V2API for Session {
     fn get_server<S: AsRef<str>>(&self, id: S) -> Result<protocol::Server> {
         trace!("Get compute server {}", id.as_ref());
-        let server = self.request::<V2>(Method::Get, &["servers", id.as_ref()])?
+        let server = self.request::<V2>(Method::Get,
+                                        &["servers", id.as_ref()],
+                                        None)?
            .receive_json::<protocol::ServerRoot>()?.server;
         trace!("Received {:?}", server);
         Ok(server)
     }
 
     fn list_servers<Q: Serialize + Debug>(&self, query: &Q)
-            -> Result<Vec<protocol::ServerSummary>> {
+            -> Result<Vec<common::protocol::IdAndName>> {
         trace!("Listing compute servers with {:?}", query);
-        let result = self.request::<V2>(Method::Get, &["servers"])?
+        let result = self.request::<V2>(Method::Get, &["servers"], None)?
            .query(query).receive_json::<protocol::ServersRoot>()?.servers;
         trace!("Received servers: {:?}", result);
         Ok(result)
@@ -91,19 +97,24 @@ impl V2API for Session {
     fn list_servers_detail<Q: Serialize + Debug>(&self, query: &Q)
             -> Result<Vec<protocol::Server>> {
         trace!("Listing compute servers with {:?}", query);
-        let result = self.request::<V2>(Method::Get, &["servers", "detail"])?
+        let result = self.request::<V2>(Method::Get,
+                                        &["servers", "detail"],
+                                        None)?
            .query(query).receive_json::<protocol::ServersDetailRoot>()?.servers;
         trace!("Received servers: {:?}", result);
         Ok(result)
     }
 
-    fn server_simple_action<S1, S2>(&self, id: S1, action: S2) -> Result<()>
-            where S1: AsRef<str>, S2: AsRef<str> {
-        trace!("Running {} on server {}", action.as_ref(), id.as_ref());
+    fn server_action_with_args<S1, S2, Q>(&self, id: S1, action: S2, args: Q)
+            -> Result<()>
+            where S1: AsRef<str>, S2: AsRef<str>, Q: Serialize + Debug {
+        trace!("Running {} on server {} with args {:?}",
+               action.as_ref(), id.as_ref(), args);
         let mut body = HashMap::new();
-        let _ = body.insert(action.as_ref(), serde_json::Value::Null);
+        let _ = body.insert(action.as_ref(), args);
         let _ = self.request::<V2>(Method::Post,
-                                   &["servers", id.as_ref(), "action"])?
+                                   &["servers", id.as_ref(), "action"],
+                                   None)?
             .json(&body).send()?;
         debug!("Successfully ran {} on server {}", action.as_ref(), id.as_ref());
         Ok(())
@@ -111,24 +122,28 @@ impl V2API for Session {
 
     fn delete_server<S: AsRef<str>>(&self, id: S) -> Result<()> {
         trace!("Deleting server {}", id.as_ref());
-        let _ = self.request::<V2>(Method::Delete, &["servers", id.as_ref()])?
+        let _ = self.request::<V2>(Method::Delete,
+                                   &["servers", id.as_ref()],
+                                   None)?
             .send()?;
-        debug!("Deleted server {}", id.as_ref());
+        debug!("Successfully requested deletion of server {}", id.as_ref());
         Ok(())
     }
 
     fn get_flavor<S: AsRef<str>>(&self, id: S) -> Result<protocol::Flavor> {
         trace!("Get compute flavor {}", id.as_ref());
-        let flavor = self.request::<V2>(Method::Get, &["flavors", id.as_ref()])?
+        let flavor = self.request::<V2>(Method::Get,
+                                        &["flavors", id.as_ref()],
+                                        None)?
            .receive_json::<protocol::FlavorRoot>()?.flavor;
         trace!("Received {:?}", flavor);
         Ok(flavor)
     }
 
     fn list_flavors<Q: Serialize + Debug>(&self, query: &Q)
-            -> Result<Vec<protocol::FlavorSummary>> {
+            -> Result<Vec<common::protocol::IdAndName>> {
         trace!("Listing compute flavors with {:?}", query);
-        let result = self.request::<V2>(Method::Get, &["flavors"])?
+        let result = self.request::<V2>(Method::Get, &["flavors"], None)?
            .query(query).receive_json::<protocol::FlavorsRoot>()?.flavors;
         trace!("Received flavors: {:?}", result);
         Ok(result)
@@ -137,7 +152,9 @@ impl V2API for Session {
     fn list_flavors_detail<Q: Serialize + Debug>(&self, query: &Q)
             -> Result<Vec<protocol::Flavor>> {
         trace!("Listing compute flavors with {:?}", query);
-        let result = self.request::<V2>(Method::Get, &["flavors", "detail"])?
+        let result = self.request::<V2>(Method::Get,
+                                        &["flavors", "detail"],
+                                        None)?
            .query(query).receive_json::<protocol::FlavorsDetailRoot>()?.flavors;
         trace!("Received flavors: {:?}", result);
         Ok(result)
@@ -151,7 +168,7 @@ impl ServiceType for V2 {
     }
 
     fn service_info(endpoint: Url, auth: &AuthMethod) -> Result<ServiceInfo> {
-        common::fetch_service_info(endpoint, auth, SERVICE_TYPE, VERSION_ID)
+        common::protocol::fetch_service_info(endpoint, auth, SERVICE_TYPE, VERSION_ID)
     }
 
     fn api_version_headers(version: ApiVersion) -> Option<Headers> {
@@ -161,5 +178,3 @@ impl ServiceType for V2 {
         Some(hdrs)
     }
 }
-
-impl ApiVersioning for V2 {}
