@@ -20,7 +20,8 @@ use fallible_iterator::{IntoFallibleIterator, FallibleIterator};
 use serde::Serialize;
 
 use super::super::{Error, Result};
-use super::super::service::{ListResources, ResourceId, ResourceIterator};
+use super::super::common::{self, FlavorRef, ListResources, Refresh, ResourceId,
+                           ResourceIterator};
 use super::super::session::Session;
 use super::super::utils::Query;
 use super::base::V2API;
@@ -38,7 +39,7 @@ pub struct Flavor<'session> {
 #[derive(Clone, Debug)]
 pub struct FlavorSummary<'session> {
     session: &'session Session,
-    inner: protocol::FlavorSummary
+    inner: common::protocol::IdAndName,
 }
 
 /// A query to server list.
@@ -49,12 +50,6 @@ pub struct FlavorQuery<'session> {
     can_paginate: bool,
 }
 
-/// Trait for something that can be used as a flavor ID.
-pub trait ToFlavorId {
-    /// Get flavor ID as a string.
-    fn to_flavor_id(&self) -> String;
-}
-
 
 impl<'session> Flavor<'session> {
     /// Load a Flavor object.
@@ -67,12 +62,6 @@ impl<'session> Flavor<'session> {
         })
     }
 
-    /// Refresh the server.
-    pub fn refresh(&mut self) -> Result<()> {
-        self.inner = self.session.get_flavor(&self.inner.id)?;
-        Ok(())
-    }
-
     /// Get ephemeral disk size in GiB.
     ///
     /// Returns `0` when ephemeral disk was not requested.
@@ -118,6 +107,14 @@ impl<'session> Flavor<'session> {
     }
 }
 
+impl<'session> Refresh for Flavor<'session> {
+    /// Refresh the flavor.
+    fn refresh(&mut self) -> Result<()> {
+        self.inner = self.session.get_flavor(&self.inner.id)?;
+        Ok(())
+    }
+}
+
 impl<'session> FlavorSummary<'session> {
     /// Get a reference to flavor unique ID.
     pub fn id(&self) -> &String {
@@ -172,6 +169,7 @@ impl<'session> FlavorQuery<'session> {
     ///
     /// Note that no requests are done until you start iterating.
     pub fn into_iter(self) -> ResourceIterator<'session, FlavorSummary<'session>> {
+        debug!("Fetching flavors with {:?}", self.query);
         ResourceIterator::new(self.session, self.query)
     }
 
@@ -185,6 +183,7 @@ impl<'session> FlavorQuery<'session> {
     ///
     /// Note that no requests are done until you start iterating.
     pub fn into_iter_detailed(self) -> ResourceIterator<'session, Flavor<'session>> {
+        debug!("Fetching flavor details with {:?}", self.query);
         ResourceIterator::new(self.session, self.query)
     }
 
@@ -194,6 +193,21 @@ impl<'session> FlavorQuery<'session> {
     pub fn all(self) -> Result<Vec<FlavorSummary<'session>>> {
         self.into_iter().collect()
     }
+
+    /// Return one and exactly one result.
+    ///
+    /// Fails with `ResourceNotFound` if the query produces no results and
+    /// with `TooManyItems` if the query produces more than one result.
+    pub fn one(mut self) -> Result<FlavorSummary<'session>> {
+        debug!("Fetching one flavor with {:?}", self.query);
+        if self.can_paginate {
+            // We need only one result. We fetch maximum two to be able
+            // to check if the query yieled more than one result.
+            self.query.push("limit", 2);
+        }
+
+        self.into_iter().one()
+    }
 }
 
 
@@ -245,26 +259,14 @@ impl<'session> IntoFallibleIterator for FlavorQuery<'session> {
     }
 }
 
-impl ToFlavorId for String {
-    fn to_flavor_id(&self) -> String {
-        self.clone()
+impl<'session> From<Flavor<'session>> for FlavorRef {
+    fn from(value: Flavor<'session>) -> FlavorRef {
+        FlavorRef::from(value.inner.id)
     }
 }
 
-impl ToFlavorId for str {
-    fn to_flavor_id(&self) -> String {
-        String::from(self)
-    }
-}
-
-impl<'session> ToFlavorId for Flavor<'session> {
-    fn to_flavor_id(&self) -> String {
-        self.inner.id.clone()
-    }
-}
-
-impl<'session> ToFlavorId for FlavorSummary<'session> {
-    fn to_flavor_id(&self) -> String {
-        self.inner.id.clone()
+impl<'session> From<FlavorSummary<'session>> for FlavorRef {
+    fn from(value: FlavorSummary<'session>) -> FlavorRef {
+        FlavorRef::from(value.inner.id)
     }
 }
