@@ -32,8 +32,8 @@ mod protocol;
 mod servers;
 
 pub use self::base::V2 as ServiceType;
-pub use self::flavors::{Flavor, FlavorSummary, FlavorQuery, ToFlavorId};
-pub use self::protocol::{AddressType, ServerAddress, ServerSortKey,
+pub use self::flavors::{Flavor, FlavorSummary, FlavorQuery};
+pub use self::protocol::{AddressType, RebootType, ServerAddress, ServerSortKey,
                          ServerPowerState, ServerStatus};
-pub use self::servers::{Server, ServerQuery, ServerStatusWaiter, ServerSummary,
-                        ToImageId};
+pub use self::servers::{Server, ServerQuery,
+                        ServerStatusWaiter, ServerSummary};
