@@ -16,7 +16,7 @@
 
 use std::collections::HashMap;
 use std::fmt::Debug;
-use std::net::{Ipv4Addr, Ipv6Addr};
+use std::net::{IpAddr, Ipv4Addr, Ipv6Addr};
 use std::time::Duration;
 
 use chrono::{DateTime, FixedOffset};
@@ -24,11 +24,12 @@ use fallible_iterator::{IntoFallibleIterator, FallibleIterator};
 use serde::Serialize;
 
 use super::super::{Error, ErrorKind, Result, Sort, Waiter};
-use super::super::service::{ListResources, ResourceId, ResourceIterator};
+use super::super::common::{self, DeletionWaiter, FlavorRef, ImageRef,
+                           ListResources, ProjectRef, Refresh, ResourceId,
+                           ResourceIterator, UserRef};
 use super::super::session::Session;
 use super::super::utils::Query;
 use super::base::V2API;
-use super::flavors::ToFlavorId;
 use super::protocol;
 
 
@@ -51,13 +52,7 @@ pub struct Server<'session> {
 #[derive(Clone, Debug)]
 pub struct ServerSummary<'session> {
     session: &'session Session,
-    inner: protocol::ServerSummary
-}
-
-/// Trait for something that can be used as an image ID.
-pub trait ToImageId {
-    /// Get flavor ID as a string.
-    fn to_image_id(&self) -> String;
+    inner: common::protocol::IdAndName
 }
 
 /// Waiter for server status to change.
@@ -68,6 +63,14 @@ pub struct ServerStatusWaiter<'server> {
 }
 
 
+impl<'session> Refresh for Server<'session> {
+    /// Refresh the server.
+    fn refresh(&mut self) -> Result<()> {
+        self.inner = self.session.get_server(&self.inner.id)?;
+        Ok(())
+    }
+}
+
 impl<'session> Server<'session> {
     /// Load a Server object.
     pub(crate) fn new<Id: AsRef<str>>(session: &'session Session, id: Id)
@@ -79,12 +82,6 @@ impl<'session> Server<'session> {
         })
     }
 
-    /// Refresh the server.
-    pub fn refresh(&mut self) -> Result<()> {
-        self.inner = self.session.get_server(&self.inner.id)?;
-        Ok(())
-    }
-
     /// Get a reference to IPv4 address.
     ///
     /// If not None, this address should be used to access the server instead
@@ -116,6 +113,16 @@ impl<'session> Server<'session> {
         &self.inner.created
     }
 
+    /// Find a floating IP, if it exists.
+    ///
+    /// If multiple floating IPs exist, the first is returned.
+    pub fn floating_ip(&self) -> Option<IpAddr> {
+        self.inner.addresses.values()
+            .flat_map(|l| l.iter())
+            .filter(|a| a.addr_type == Some(protocol::AddressType::Floating))
+            .map(|a| a.addr).next()
+    }
+
     /// Get a reference to server unique ID.
     pub fn id(&self) -> &String {
         &self.inner.id
@@ -158,6 +165,23 @@ impl<'session> Server<'session> {
         &self.inner.updated
     }
 
+    /// Delete the server.
+    pub fn delete(self) -> Result<DeletionWaiter<Server<'session>>> {
+        self.session.delete_server(&self.inner.id)?;
+        Ok(DeletionWaiter::new(self, Duration::new(120, 0), Duration::new(1, 0)))
+    }
+
+    /// Reboot the server.
+    pub fn reboot(&'session mut self, reboot_type: protocol::RebootType) -> Result<ServerStatusWaiter<'session>> {
+        let mut args = HashMap::new();
+        let _ = args.insert("type", reboot_type);
+        self.session.server_action_with_args(&self.inner.id, "reboot", args)?;
+        Ok(ServerStatusWaiter {
+            server: self,
+            target: protocol::ServerStatus::Active
+        })
+    }
+
     /// Start the server, optionally wait for it to be active.
     pub fn start(&'session mut self) -> Result<ServerStatusWaiter<'session>> {
         self.session.server_simple_action(&self.inner.id, "os-start")?;
@@ -175,12 +199,6 @@ impl<'session> Server<'session> {
             target: protocol::ServerStatus::ShutOff
         })
     }
-
-    /// Delete the server.
-    pub fn delete(self) -> Result<()> {
-        // TODO(dtantsur): implement wait
-        self.session.delete_server(&self.inner.id)
-    }
 }
 
 impl<'server> ServerStatusWaiter<'server> {
@@ -220,7 +238,7 @@ impl<'server> Waiter<()> for ServerStatusWaiter<'server> {
                                    self.server.id())))
         } else {
             trace!("Still waiting for server {} to get to state {}, current is {}",
-                   self.server.id(), self.server.status(), self.target);
+                   self.server.id(), self.target, self.server.status());
             Ok(None)
         }
     }
@@ -285,14 +303,14 @@ impl<'session> ServerQuery<'session> {
     }
 
     /// Filter by IPv4 address that should be used to access the server.
-    pub fn with_access_ip_v4(mut self, value: Ipv4Addr) -> Self {
-        self.query.push("access_ip_v4", value);
+    pub fn with_access_ip_v4<T: Into<Ipv4Addr>>(mut self, value: T) -> Self {
+        self.query.push("access_ip_v4", value.into());
         self
     }
 
     /// Filter by IPv6 address that should be used to access the server.
-    pub fn with_access_ip_v6(mut self, value: Ipv6Addr) -> Self {
-        self.query.push("access_ipv6", value);
+    pub fn with_access_ip_v6<T: Into<Ipv6Addr>>(mut self, value: T) -> Self {
+        self.query.push("access_ipv6", value.into());
         self
     }
 
@@ -303,8 +321,8 @@ impl<'session> ServerQuery<'session> {
     }
 
     /// Filter by flavor.
-    pub fn with_flavor<T: ToFlavorId>(mut self, value: T) -> Self {
-        self.query.push_str("flavor", value.to_flavor_id());
+    pub fn with_flavor<T: Into<FlavorRef>>(mut self, value: T) -> Self {
+        self.query.push_str("flavor", value.into());
         self
     }
 
@@ -315,20 +333,20 @@ impl<'session> ServerQuery<'session> {
     }
 
     /// Filter by image ID.
-    pub fn with_image<T: ToImageId>(mut self, value: T) -> Self {
-        self.query.push_str("image", value.to_image_id());
+    pub fn with_image<T: Into<ImageRef>>(mut self, value: T) -> Self {
+        self.query.push_str("image", value.into());
         self
     }
 
     /// Filter by an IPv4 address.
-    pub fn with_ip_v4(mut self, value: Ipv4Addr) -> Self {
-        self.query.push("ip", value);
+    pub fn with_ip_v4<T: Into<Ipv4Addr>>(mut self, value: T) -> Self {
+        self.query.push("ip", value.into());
         self
     }
 
     /// Filter by an IPv6 address.
-    pub fn with_ip_v6(mut self, value: Ipv6Addr) -> Self {
-        self.query.push("ip6", value);
+    pub fn with_ip_v6<T: Into<Ipv6Addr>>(mut self, value: T) -> Self {
+        self.query.push("ip6", value.into());
         self
     }
 
@@ -338,27 +356,21 @@ impl<'session> ServerQuery<'session> {
         self
     }
 
-    /// Filter by power state.
-    pub fn with_power_state<T: Into<String>>(mut self, value: T) -> Self {
-        self.query.push_str("power_state", value);
-        self
-    }
-
     /// Filter by project ID (also commonly known as tenant ID).
-    pub fn with_project_id<T: Into<String>>(mut self, value: T) -> Self {
-        self.query.push_str("project_id", value);
+    pub fn with_project<T: Into<ProjectRef>>(mut self, value: T) -> Self {
+        self.query.push_str("project_id", value.into());
         self
     }
 
     /// Filter by server status.
-    pub fn with_status<T: Into<String>>(mut self, value: T) -> Self {
-        self.query.push_str("status", value);
+    pub fn with_status(mut self, value: protocol::ServerStatus) -> Self {
+        self.query.push_str("status", value.to_string());
         self
     }
 
     /// Filter by user ID.
-    pub fn with_user_id<T: Into<String>>(mut self, value: T) -> Self {
-        self.query.push_str("user_id", value);
+    pub fn with_user<T: Into<UserRef>>(mut self, value: T) -> Self {
+        self.query.push_str("user_id", value.into());
         self
     }
 
@@ -372,6 +384,7 @@ impl<'session> ServerQuery<'session> {
     ///
     /// Note that no requests are done until you start iterating.
     pub fn into_iter(self) -> ResourceIterator<'session, ServerSummary<'session>> {
+        debug!("Fetching servers with {:?}", self.query);
         ResourceIterator::new(self.session, self.query)
     }
 
@@ -385,6 +398,7 @@ impl<'session> ServerQuery<'session> {
     ///
     /// Note that no requests are done until you start iterating.
     pub fn into_iter_detailed(self) -> ResourceIterator<'session, Server<'session>> {
+        debug!("Fetching server details with {:?}", self.query);
         ResourceIterator::new(self.session, self.query)
     }
 
@@ -400,23 +414,14 @@ impl<'session> ServerQuery<'session> {
     /// Fails with `ResourceNotFound` if the query produces no results and
     /// with `TooManyItems` if the query produces more than one result.
     pub fn one(mut self) -> Result<ServerSummary<'session>> {
+        debug!("Fetching one server with {:?}", self.query);
         if self.can_paginate {
             // We need only one result. We fetch maximum two to be able
             // to check if the query yieled more than one result.
             self.query.push("limit", 2);
         }
 
-        let mut iter = self.into_iter();
-        match iter.next()? {
-            Some(result) => if iter.next()?.is_some() {
-                Err(Error::new(ErrorKind::TooManyItems,
-                               "Query returned more than one result"))
-            } else {
-                Ok(result)
-            },
-            None => Err(Error::new(ErrorKind::ResourceNotFound,
-                                   "Query returned no results"))
-        }
+        self.into_iter().one()
     }
 }
 
@@ -467,15 +472,3 @@ impl<'session> IntoFallibleIterator for ServerQuery<'session> {
         self.into_iter()
     }
 }
-
-impl ToImageId for String {
-    fn to_image_id(&self) -> String {
-        self.clone()
-    }
-}
-
-impl ToImageId for str {
-    fn to_image_id(&self) -> String {
-        String::from(self)
-    }
-}
