@@ -18,17 +18,14 @@
 
 use std::cell::{Ref, RefCell};
 use std::collections::HashMap;
+use std::fmt;
 use std::hash::Hash;
 
-use serde::{Deserialize, Deserializer};
-use serde::de::{DeserializeOwned, Error as DeserError};
-use serde_json;
-
 use super::{ErrorKind, Result};
 
 
 /// Type of query parameters.
-#[derive(Clone, Debug)]
+#[derive(Clone)]
 pub struct Query(pub Vec<(String, String)>);
 
 /// Cached clone-able value.
@@ -40,6 +37,12 @@ pub struct ValueCache<T: Clone>(RefCell<Option<T>>);
 pub struct MapCache<K: Hash + Eq, V: Clone>(RefCell<HashMap<K, V>>);
 
 
+impl fmt::Debug for Query {
+    fn fmt(&self, f: &mut fmt::Formatter) -> ::std::result::Result<(), fmt::Error> {
+        write!(f, "{:?}", self.0)
+    }
+}
+
 impl Query {
     /// Empty query.
     pub fn new() -> Query {
@@ -76,10 +79,39 @@ impl<T: Clone> ValueCache<T> {
         Ok(())
     }
 
-    /// Get the cached value.
-    #[inline]
-    pub fn get(&self) -> Option<T> {
-        self.0.borrow().clone()
+    /// Ensure that the cached value is valid.
+    ///
+    /// Returns `true` if the value exists and passes the check.
+    pub fn validate<F>(&self, check: F) -> bool
+            where F: FnOnce(&T) -> bool {
+        let valid = match self.0.borrow().as_ref() {
+            Some(v) => check(v),
+            None => false
+        };
+
+        if ! valid {
+            *self.0.borrow_mut() = None;
+            false
+        } else {
+            true
+        }
+    }
+
+    /// Validate value and set it if it is not valid.
+    pub fn validate_and_ensure_value<V, F>(&self, check: V, default: F) -> Result<()>
+            where V: FnOnce(&T) -> bool,
+                  F: FnOnce() -> Result<T> {
+        if self.validate(check) {
+            Ok(())
+        } else {
+            self.ensure_value(default)
+        }
+    }
+
+    /// Extract a part of the value.
+    pub fn extract<F, R>(&self, filter: F) -> Option<R>
+            where F: FnOnce(&T) -> R {
+        self.0.borrow().as_ref().map(filter)
     }
 }
 
@@ -114,33 +146,6 @@ impl<K: Hash + Eq, V: Clone> MapCache<K, V> {
     }
 }
 
-/// Deserialize value where empty string equals None.
-pub fn empty_as_none<'de, D, T>(des: D) -> ::std::result::Result<Option<T>, D::Error>
-        where D: Deserializer<'de>, T: DeserializeOwned {
-    let value = serde_json::Value::deserialize(des)?;
-    match &value {
-        &serde_json::Value::String(ref s) if s == "" => return Ok(None),
-        _ => ()
-    };
-
-    match serde_json::from_value(value) {
-        Ok(value) => Ok(Some(value)),
-        Err(e) => Err(DeserError::custom(e))
-    }
-}
-
-/// Deserialize value where empty string equals None.
-pub fn empty_as_default<'de, D, T>(des: D) -> ::std::result::Result<T, D::Error>
-        where D: Deserializer<'de>, T: DeserializeOwned + Default {
-    let value = serde_json::Value::deserialize(des)?;
-    match &value {
-        &serde_json::Value::String(ref s) if s == "" =>
-            return Ok(Default::default()),
-        _ => ()
-    };
-
-    serde_json::from_value(value).map_err(DeserError::custom)
-}
 
 /// Extensions for Result type.
 pub trait ResultExt<T> {
@@ -208,10 +213,10 @@ pub mod test {
     use reqwest::{IntoUrl, Url};
     use reqwest::header::Headers;
 
-    use super::super::{Error, ErrorKind, Result, ApiVersion};
+    use super::super::{Error, ErrorKind, Result};
     use super::super::auth::{AuthMethod, NoAuth};
-    use super::super::service::{ServiceInfo, ServiceType};
-    use super::super::session::Session;
+    use super::super::common::ApiVersion;
+    use super::super::session::{Session, ServiceInfo, ServiceType};
 
     /// Create a session with fake authentication.
     pub fn new_session<U: IntoUrl>(endpoint: U) -> Session {
